#include <stdio.h>
#include <cstring>

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "audio.h"

#include "screens.h"
#include "visuals.h"

extern Screens* screens;
extern Visuals* visuals;

//-------------------------------------------------------------------------------------------------
Audio::Audio(void)
{
    MusicVolume = 64;
    SoundVolume = 64;
    CurrentlySelectedMusicTrack = 0;

    for (int soundIndex = 0; soundIndex < SoundTotal; soundIndex++)  SoundFX[soundIndex] = NULL;
    for (int musicIndex = 0; musicIndex < MusicTotal; musicIndex++)  MusicTrack[musicIndex] = NULL;

    AudioWorking = false;
}

//-------------------------------------------------------------------------------------------------
Audio::~Audio(void)
{
    if (AudioWorking == false)  return;

    Mix_HaltChannel(-1);
    for (int soundIndex = 0; soundIndex < SoundTotal; soundIndex++)
    {
        if (SoundFX[soundIndex] != NULL)  Mix_FreeChunk(SoundFX[soundIndex]);
        SoundFX[soundIndex] = NULL;
    }
    Mix_AllocateChannels(0);
    printf("Unloaded all music from memory.\n");

    Mix_HaltMusic();
    for (int musicIndex = 0; musicIndex < MusicTotal; musicIndex++)
    {
        if (MusicTrack[musicIndex] != NULL)  Mix_FreeMusic(MusicTrack[musicIndex]);
        MusicTrack[musicIndex] = NULL;
    }
    printf("Unloaded all sounds from memory.\n");

    Mix_CloseAudio();
    Mix_Quit();
    printf("SDL_mixer closed.\n");
}

//-------------------------------------------------------------------------------------------------
void Audio::SetupAudio(void)
{
    strcpy(Mix_Init_Error, "SDL2_Mixer: OK");

    int flags = MIX_INIT_OGG|MIX_INIT_MODPLUG;
    int initted = Mix_Init(flags);
    if ( (initted&flags) != flags )
    {
        printf("Mix_Init: Failed to init required ogg and mod support!\n");
        printf("Mix_Init: %s\n", Mix_GetError());

        strcpy(Mix_Init_Error, "SDL2_Mixer ERROR: ");
        strcat( Mix_Init_Error, Mix_GetError() );

        AudioWorking = false;
        return;
    }

    if (Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 1/*MIX_DEFAULT_CHANNELS*/, 2048)==-1)
    {
        printf("ERROR: SDL2_Mixer init failed: %s\n", Mix_GetError());

        strcpy(Mix_Init_Error, "SDL2_Mixer ERROR: ");
        strcat( Mix_Init_Error, Mix_GetError() );

        AudioWorking = false;
        return;
    }
    else printf("SDL2_Mixer initialized.\n");

    Mix_AllocateChannels(16);
    for (int channel = 0; channel < 16; channel++)
    {
        Mix_Volume(channel, SoundVolume);
    }

    for (int index = 0; index < MusicTotal; index++)
    {
        switch(index)
        {
            case 0:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-01.mod");
                break;

            case 1:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-02.mod");
                break;

            case 2:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-03.mod");
                break;

            case 3:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-04.mod");
                break;

            case 4:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-05.mod");
                break;

            case 5:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-06.mod");
                break;

            case 6:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-07.mod");
                break;

            case 7:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-08.mod");
                break;

            case 8:
                MusicTrack[index] = Mix_LoadMUS("data/audio/Track-09.mod");
                break;


            default:
                break;
        }

        if(!MusicTrack[index] && MusicTotal != 0)
        {
            printf("ERROR: Couldn't load music %s\n", Mix_GetError());
        }
        else  Mix_VolumeMusic(MusicVolume);
    }
    printf("Loaded all music into memory.\n");

    for (int index = 0; index < SoundTotal; index++)
    {
        switch(index)
        {
            case 0:
                SoundFX[index] = Mix_LoadWAV("data/audio/Click.wav");
                break;

            case 1:
                SoundFX[index] = Mix_LoadWAV("data/audio/Bomb-Explosion.wav");
                break;

            case 2:
                SoundFX[index] = Mix_LoadWAV("data/audio/Undo.wav");
                break;

            case 3:
                SoundFX[index] = Mix_LoadWAV("data/audio/Clear.wav");
                break;

            case 4:
                SoundFX[index] = Mix_LoadWAV("data/audio/Bomb-Launched.wav");
                break;

            case 5:
                SoundFX[index] = Mix_LoadWAV("data/audio/Word-Good.wav");
                break;

            case 6:
                SoundFX[index] = Mix_LoadWAV("data/audio/Word-Bad.wav");
                break;

            case 7:
                SoundFX[index] = Mix_LoadWAV("data/audio/Level-Up.wav");
                break;

            case 8:
                SoundFX[index] = Mix_LoadWAV("data/audio/Bomb-Up.wav");
                break;

            case 9:
                SoundFX[index] = Mix_LoadWAV("data/audio/Cheer.wav");
                break;


            default:
                break;
        }

        if(!SoundFX[index])
        {
            printf("ERROR: Couldn't load sound effect %s\n", Mix_GetError());
        }
        else  Mix_VolumeChunk(SoundFX[index], MIX_MAX_VOLUME);
    }
    printf("Loaded all sounds into memory.\n");

    AudioWorking = true;
}

//-------------------------------------------------------------------------------------------------
void Audio::PlayMusic(Uint8 musicIndex, int loop)
{
    if (AudioWorking == false)  return;

    if (MusicTrack[musicIndex] == NULL)  return;

    if (musicIndex >= MusicTotal)  return;

    if (Mix_PlayingMusic() == 1)  Mix_HaltMusic();

    Mix_VolumeMusic(MusicVolume);

    CurrentlySelectedMusicTrack = musicIndex;

    if(Mix_PlayMusic(MusicTrack[musicIndex], loop)==-1)
    {
        printf( "Mix_PlayMusic: %s\n", Mix_GetError() );
    }
/*
    switch(CurrentlySelectedMusicTrack)
    {
        case 0:
            SDL_SetWindowTitle(visuals->Window, "t.A.T.u. - ''All About Us''");
            break;

        default:
            break;
    }

    if (MusicVolume == 0)  SDL_SetWindowTitle(visuals->Window, "''TetriCrisis 4 110% A.I. Turbo'' - (C)2014 By 16BitSoft Inc."); */
}

//-------------------------------------------------------------------------------------------------
void Audio::PlayDigitalSoundFX(Uint8 soundIndex, int loops)
{
    if (AudioWorking == false)  return;

    if (SoundFX[soundIndex] == NULL)  return;

    if (soundIndex >= SoundTotal)  return;

    if (SoundVolume == 0)  return;

    for (int index = 1; index < 16; index++)
        Mix_Volume( index, SoundVolume );

    for (int indexTwo = 0; indexTwo < SoundTotal; indexTwo++)
        Mix_VolumeChunk(SoundFX[indexTwo], SoundVolume);

    if(Mix_PlayChannel(-1, SoundFX[soundIndex], loops)==-1)
    {
        printf( "Mix_PlayChannel: %s\n",Mix_GetError() );
    }
}
//-------------------------------------------------------------------------------------------------
