/*___________________________________________________________________________________________________________________________
                                                      JeZxLee's
                                                      SDL 2.0.3
                                             Open Source / Cross Platform
  _______ _______     ______   _______    _       _______          _           _______ _                                _ TM
 (_______|_______)   (_____ \ (_______)  (_)     (_______)        | |         (_______) |                              | |
  _   ___    _  _____ _____) )    _ _ _ _ _ ____     _ _   _  ____| |__   ___  _      | |__  _____  ____ ____ _____  __| |
 | | (_  |  | |(_____)  __  /    | | | | | |  _ \   | | | | |/ ___)  _ \ / _ \| |     |  _ \(____ |/ ___) _  | ___ |/ _  |
 | |___) |  | |      | |  \ \    | | | | | | | | |  | | |_| | |   | |_) ) |_| | |_____| | | / ___ | |  ( (_| | ____( (_| |
  \_____/   |_|      |_|   |_|   |_|\___/|_|_| |_|  |_|____/|_|   |____/ \___/ \______)_| |_\_____|_|   \___ |_____)\____|
                                                                                                       (_____|
                                           2-Dimensional Video Game Engine

                          (C)opyright 2015 By Jesse "JeZxLee" Palser (JessePalser@GMail.com)
.............................................................................................................................

                                                  "LettersFall[TM]"

                                                        Beta1

                                        (C)opyright 2015 By Team 16BitSoft Inc.
                                                  www.16BitSoft.com
_____________________________________________________________________________________________________________________________*/
#define SDL_MAIN_HANDLED
#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#include <SDL.h>
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "visuals.h"
#include "input.h"
#include "screens.h"
#include "interface.h"
#include "audio.h"
#include "data.h"
#include "logic.h"

Visuals *visuals;
Input *input;
Screens *screens;
Interface *interface;
Audio *audio;
Data *data;
Logic *logic;


//-------------------------------------------------------------------------------------------------
int main( int argc, char* args[] )
{
    printf("JeZxLee's ''GT-R Twin TurboCharged'' game engine started!\n");
    argc = argc; args = args;

    if ( SDL_Init(SDL_INIT_EVERYTHING) != 0 )
    {
        printf( "Unable to initialize SDL2: %s\n", SDL_GetError() );
        return(1);
    }
    else  printf("SDL2 initialized.\n");

    visuals = new Visuals();

    if ( visuals->InitializeWindow() != true ) visuals->CoreFailure = true;
    if ( visuals->LoadFontsIntoMemory() != true )  visuals->CoreFailure = true;
    if ( visuals->LoadSpritesAndInitialize() != true )  visuals->CoreFailure = true;

    input = new Input();

    screens = new Screens();

    interface = new Interface();

    data = new Data();

    srand( (unsigned)time(NULL) ); // Place unique time seed into random number generator.
    logic = new Logic();

    audio = new Audio();
    audio->SetupAudio();

    data->LoadHighScoresAndOptions();

    if (visuals->FullScreenMode == true)  SDL_SetWindowFullscreen(visuals->Window, SDL_WINDOW_FULLSCREEN_DESKTOP);

    audio->PlayMusic(logic->SelectedMusicTrack, 0);

    //-MAIN-LOOP------------------------------------------------------------------------
    printf("Main loop started...\n");
    while (visuals->CoreFailure != true && input->EXIT_Game != true)
    {
        input->GetAllUserInput();
        visuals->CalculateFramerate();
        screens->ProcessScreenToDisplay();
        visuals->ProcessFramerate();
    }
    printf("...Main loop exited\n");
    //------------------------------------------------------------------------MAIN-LOOP-

    data->SaveHighScoresAndOptions();

    delete logic;
    delete data;
    delete audio;
    delete interface;
    delete screens;
    delete input;
    delete visuals;
    SDL_Quit();
    printf("SDL2 closed.\n");
    printf("JeZxLee's ''GT-R Twin TurboCharged'' game engine ended!\n");
    return(0);
}

//                    TM
// "A 110% By JeZxLee!"
