#include <stdio.h>
#include <cstring>

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "visuals.h"

#include "screens.h"

extern Screens* screens;

//-------------------------------------------------------------------------------------------------
Visuals::Visuals(void)
{
    CoreFailure = false;

    ForceAspectRatio = false;

    FullScreenMode = false;

    FrameLock = 16;
    SystemTicks = SDL_GetTicks();
    NextFrameTicks = SystemTicks + FrameLock;
    NumberOfFrames = 0;

    CurrentFramePerSecond = 0;

    for (int index = 0; index < 10; index++)
    {
        FramesPerSecondArray[index] = 0;
    }

    NextSecondTick = 0;
    AverageFPS = 0;

    TotalNumberOfLoadedStaffTexts = 0;

    TextCacheCurrentIndex = 0;

	for (int index = 0; index < NumberOfTextsCached; index++)
	{
		TextTexture[index] = NULL;
		TextOutlineTexture[index] = NULL;
	}
    ClearTextCache();

    if (TTF_Init()==-1)
    {
        printf( "SDL2_TTF initialization failed: %s\n", TTF_GetError() );
        CoreFailure = true;
    }
}

//-------------------------------------------------------------------------------------------------
Visuals::~Visuals(void)
{
    ClearTextCache();

    UnloadFontsFromMemory();

    TTF_Quit();

    for (int index = 0; index < NumberOfSprites; index++)
    {
        if (Sprites[index].Texture != NULL)
        {
            SDL_DestroyTexture(Sprites[index].Texture);
        }
    }
    printf("Unloaded all sprite images from memory.\n");

    SDL_DestroyRenderer(Renderer);
    printf("SDL2 renderer destroyed.\n");

    SDL_DestroyWindow(Window);
    printf("SDL2 window destroyed.\n");
}

//-------------------------------------------------------------------------------------------------
void Visuals::CalculateFramerate(void)
{
    SystemTicks = SDL_GetTicks();
    NextFrameTicks = SystemTicks + FrameLock;

    NumberOfFrames++;

    if (SystemTicks > NextSecondTick)
    {
        NextSecondTick = SystemTicks + 1000;

        FramesPerSecondArray[CurrentFramePerSecond] = NumberOfFrames;
        NumberOfFrames = 0;

        if (CurrentFramePerSecond < 9)  CurrentFramePerSecond++;
        else  CurrentFramePerSecond = 0;

        Uint32 frameTotal = 0;
        for (int index = 0; index < 10; index++)
        {
            frameTotal += FramesPerSecondArray[index];
        }

        AverageFPS = frameTotal / 10;
    }
}

//-------------------------------------------------------------------------------------------------
void Visuals::ProcessFramerate(void)
{
    SystemTicks = SDL_GetTicks();
    if (NextFrameTicks > SystemTicks)  SDL_Delay(NextFrameTicks - SystemTicks);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::InitializeWindow(void)
{
SDL_Surface* windowIcon = SDL_LoadBMP("data/visuals/icon.bmp");

    Window = NULL;
    Window = SDL_CreateWindow("''LettersFall[TM]'' - (C)2015 By 16BitSoft Inc."
                                 , SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, 640, 480, SDL_WINDOW_RESIZABLE);

    if (Window == NULL)
    {
        printf( "SDL2 create window failed: %s\n", SDL_GetError() );
        CoreFailure = true;
        return(false);
    }
    else  printf("SDL2 window created.\n");

    WindowWidthCurrent = 640;
    WindowHeightCurrent = 480;

    SDL_SetWindowIcon(Window, windowIcon);

    Renderer = NULL;
    Renderer = SDL_CreateRenderer(Window, -1, SDL_RENDERER_ACCELERATED);

    if (Renderer == NULL)
    {
        printf( "SDL2 create renderer failed: %s\n", SDL_GetError() );
        CoreFailure = true;
        return(false);
    }
    else  printf("SDL2 renderer created.\n");

    return(true);
}

//-------------------------------------------------------------------------------------------------
void Visuals::ClearScreenBufferWithColor(Uint8 red, Uint8 green, Uint8 blue, Uint8 alpha)
{
    SDL_SetRenderDrawColor(Renderer, red, green, blue, alpha);
    SDL_RenderFillRect(Renderer, NULL);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadSpritesAndInitialize(void)
{
char filePath[256];
SDL_Surface *tempSurface = NULL;
Uint32 textureFormat;
int textureAccess;
int textureWidth;
int textureHeight;

    for (int index = 0; index < NumberOfSprites; index++)
    {
        Sprites[index].Texture = NULL;
    }

    for (int index = 0; index < NumberOfSprites; index++)
    {
        strcpy(filePath, "~");

        switch(index)
        {
            case 0:
                strcpy(filePath, "data/visuals/Screen-Fade-Black-Box.png");
                break;

            case 1:
                strcpy(filePath, "data/visuals/16BitSoft-Logo.png");
                break;

            case 2:
                strcpy(filePath, "data/visuals/Waterfall-BG.png");
                break;

            case 3:
                strcpy(filePath, "data/visuals/LettersFall-Text-Logo.png");
                break;

            case 4:
                strcpy(filePath, "data/visuals/How-To-Play.png");
                break;

            case 10:
                strcpy(filePath, "data/visuals/SDL2-Logo.png");
                break;


            case 20:
                strcpy(filePath, "data/visuals/Waterfall-BG.png");
                break;

            case 21:
                strcpy(filePath, "data/visuals/Water-BG.png");
                break;

            case 22:
                strcpy(filePath, "data/visuals/Kittens-BG.png");
                break;

            case 23:
                strcpy(filePath, "data/visuals/Puppies-BG.png");
                break;

            case 24:
                strcpy(filePath, "data/visuals/Chalkboard-BG.png");
                break;

            case 25:
                strcpy(filePath, "data/visuals/Sunset-BG.png");
                break;


            case 40:
                strcpy(filePath, "data/visuals/LettersFall-Text-Logo.png");
                break;

            case 55:
                strcpy(filePath, "data/visuals/X-Button.png");
                break;

            case 60:
                strcpy(filePath, "data/visuals/Playfield-Board.png");
                break;

            case 65:
                strcpy(filePath, "data/visuals/Next-Text.png");
                break;

            case 70:
                strcpy(filePath, "data/visuals/Level-Text.png");
                break;

            case 75:
                strcpy(filePath, "data/visuals/Score-Text.png");
                break;

            case 80:
                strcpy(filePath, "data/visuals/Bomb-Bonus-Meter.png");
                break;

            case 85:
                strcpy(filePath, "data/visuals/Bomb-Bonus-Meter-Green-Line.png");
                break;

            case 90:
                strcpy(filePath, "data/visuals/Bomb.png");
                break;

            case 95:
                strcpy(filePath, "data/visuals/Bomb-Button.png");
                break;

            case 100:
                strcpy(filePath, "data/visuals/Left-Arrow.png");
                break;

            case 105:
                strcpy(filePath, "data/visuals/Down-Arrow.png");
                break;

            case 110:
                strcpy(filePath, "data/visuals/Right-Arrow.png");
                break;

            case 115:
                strcpy(filePath, "data/visuals/Clear-Button.png");
                break;

            case 120:
                strcpy(filePath, "data/visuals/Check-Word-Button.png");
                break;

            case 125:
                strcpy(filePath, "data/visuals/Undo-Button.png");
                break;

            case 130:
                strcpy(filePath, "data/visuals/Bomb-Explosion.png");
                break;

            case 135:
                strcpy(filePath, "data/visuals/Game-Over-Text.png");
                break;


            default:
                break;
        }

        if (filePath[0] != '~')
        {
            tempSurface = IMG_Load(filePath);

            if (!tempSurface)
            {
                printf( "Image loading failed: %s\n", IMG_GetError() );
                CoreFailure = true;
                return(false);
            }
        }

        if (filePath[0] != '~')
        {
            Sprites[index].Texture = SDL_CreateTextureFromSurface(Renderer, tempSurface);

            Sprites[index].ScreenX = 320;
            Sprites[index].ScreenY = 240;
            Sprites[index].ScaleX = 1.0;
            Sprites[index].ScaleY = 1.0;
            Sprites[index].RotationDegree = 0;
            Sprites[index].RedHue = 255;
            Sprites[index].GreenHue = 255;
            Sprites[index].BlueHue = 255;
            Sprites[index].Transparency = 255;
            Sprites[index].Smooth = false;
            Sprites[index].FlipX = false;
            Sprites[index].FlipY = false;

            Sprites[index].AnimationTimer = 0;

            SDL_QueryTexture(Sprites[index].Texture, &textureFormat, &textureAccess, &textureWidth, &textureHeight);
            Sprites[index].TextureWidthOriginal = textureWidth;
            Sprites[index].TextureHeightOriginal = textureHeight;

            SDL_FreeSurface(tempSurface);
        }
    }

    if ( LoadInterfaceIntoMemory() == false)
    {
        CoreFailure = true;
        return(false);
    }

    if ( PreloadStaffTextsIntoMemory() == false)
    {
        CoreFailure = true;
        return(false);
    }

    printf("Loaded all sprite images into memory.\n");
    return(true);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadInterfaceIntoMemory(void)
{
char filePath[256];
SDL_Surface *tempSurface = NULL;
Uint32 textureFormat;
int textureAccess;
int textureWidth;
int textureHeight;

    for (int index = 1000; index < 1010; index++)
    {
        Sprites[index].Texture = NULL;
    }

    for (int index = 1000; index < 1010; index++)
    {
        strcpy(filePath, "~");

        switch(index)
        {
            case 1000:
                strcpy(filePath, "data/visuals/Button-Selector-Left.png");
                break;

            case 1001:
                strcpy(filePath, "data/visuals/Button-Selector-Right.png");
                break;

            case 1002:
                strcpy(filePath, "data/visuals/Selector-Line.png");
                break;

            case 1003:
                strcpy(filePath, "data/visuals/Button.png");
                break;

            case 1004:
                strcpy(filePath, "data/visuals/Button.png");
                break;

            case 1005:
                strcpy(filePath, "data/visuals/Button.png");
                break;

            case 1006:
                strcpy(filePath, "data/visuals/Button.png");
                break;

            case 1007:
                strcpy(filePath, "data/visuals/Button.png");
                break;

            case 1008:
                strcpy(filePath, "data/visuals/Button.png");
                break;

            case 1009:
                strcpy(filePath, "data/visuals/Button.png");
                break;


            default:
                break;
        }

        if (filePath[0] != '~')
        {
            tempSurface = IMG_Load(filePath);

            if (!tempSurface)
            {
                printf( "Image loading failed: %s\n", IMG_GetError() );
                CoreFailure = true;
                return(false);
            }
        }

        if (index > 1002 && index < 1010)
        {
            char buttonText[64];

            strcpy(buttonText, " \0");

            if      (index == 1003)  strcpy(buttonText, "START!\0");
            else if (index == 1004)  strcpy(buttonText, "Options\0");
            else if (index == 1005)  strcpy(buttonText, "How To Play\0");
            else if (index == 1006)  strcpy(buttonText, "High Scores\0");
            else if (index == 1007)  strcpy(buttonText, "About\0");
            else if (index == 1008)  strcpy(buttonText, "Exit\0");
            else if (index == 1009)  strcpy(buttonText, "Back\0");

            SDL_Color textColor = { 0, 0, 0, 255 };
            SDL_Color outlineColor = { 208, 132, 17, 255 };
            SDL_Surface *text;
            SDL_Surface *textOutline;
            SDL_Rect destRect;

            text = TTF_RenderText_Blended(Font[0], buttonText, textColor);
            textOutline = TTF_RenderText_Solid(Font[0], buttonText, outlineColor);

            for (int posY = -2; posY < 3; posY++)
            {
                for (int posX = -2; posX < 3; posX++)
                {
                    destRect.x = (tempSurface->w / 2) - (text->w / 2) + posX;
                    destRect.y = (tempSurface->h / 2) - (text->h / 2) + posY;

                    SDL_BlitSurface(textOutline, NULL, tempSurface, &destRect);
                }
            }

            destRect.x = (tempSurface->w / 2) - (text->w / 2);
            destRect.y = (tempSurface->h / 2) - (text->h / 2);

            SDL_BlitSurface(text, NULL, tempSurface, &destRect);

            SDL_FreeSurface(text);
            SDL_FreeSurface(textOutline);
        }

        if (filePath[0] != '~')
        {
            Sprites[index].Texture = SDL_CreateTextureFromSurface(Renderer, tempSurface);

            Sprites[index].ScreenX = 320;
            Sprites[index].ScreenY = 240;
            Sprites[index].ScaleX = 1.0;
            Sprites[index].ScaleY = 1.0;
            Sprites[index].RotationDegree = 0;
            Sprites[index].RedHue = 255;
            Sprites[index].GreenHue = 255;
            Sprites[index].BlueHue = 255;
            Sprites[index].Transparency = 255;
            Sprites[index].Smooth = false;
            Sprites[index].FlipX = false;
            Sprites[index].FlipY = false;

            Sprites[index].AnimationTimer = 0;

            SDL_QueryTexture(Sprites[index].Texture, &textureFormat, &textureAccess, &textureWidth, &textureHeight);
            Sprites[index].TextureWidthOriginal = textureWidth;
            Sprites[index].TextureHeightOriginal = textureHeight;

            SDL_FreeSurface(tempSurface);
        }
    }

    if ( LoadCharacterTilesIntoMemory() == false)  return(false);

    return(true);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadCharacterTilesIntoMemory(void)
{
char filePath[256];
SDL_Surface *tempSurface = NULL;
Uint32 textureFormat;
int textureAccess;
int textureWidth;
int textureHeight;

    for (int index = 300; index < 373; index++)
    {
        Sprites[index].Texture = NULL;
    }

    for (int index = 300; index < 373; index++)
    {
        strcpy(filePath, "~");
        strcpy(filePath, "data/visuals/Letter-Tile.png");

        if (filePath[0] != '~')
        {
            tempSurface = IMG_Load(filePath);

            if (!tempSurface)
            {
                printf( "Image loading failed: %s\n", IMG_GetError() );
                CoreFailure = true;
                return(false);
            }
        }

        if (index > 299 && index < 373)
        {
            char buttonText[64];

            strcpy(buttonText, " \0");

            switch(index)
            {
                case 300:
                    strcpy(buttonText, "A\0");
                    break;

                case 301:
                    strcpy(buttonText, "B\0");
                    break;

                case 302:
                    strcpy(buttonText, "C\0");
                    break;

                case 303:
                    strcpy(buttonText, "D\0");
                    break;

                case 304:
                    strcpy(buttonText, "E\0");
                    break;

                case 305:
                    strcpy(buttonText, "F\0");
                    break;

                case 306:
                    strcpy(buttonText, "G\0");
                    break;

                case 307:
                    strcpy(buttonText, "H\0");
                    break;

                case 308:
                    strcpy(buttonText, "I\0");
                    break;

                case 309:
                    strcpy(buttonText, "J\0");
                    break;

                case 310:
                    strcpy(buttonText, "K\0");
                    break;

                case 311:
                    strcpy(buttonText, "L\0");
                    break;

                case 312:
                    strcpy(buttonText, "M\0");
                    break;

                case 313:
                    strcpy(buttonText, "N\0");
                    break;

                case 314:
                    strcpy(buttonText, "O\0");
                    break;

                case 315:
                    strcpy(buttonText, "P\0");
                    break;

                case 316:
                    strcpy(buttonText, "Q\0");
                    break;

                case 317:
                    strcpy(buttonText, "R\0");
                    break;

                case 318:
                    strcpy(buttonText, "S\0");
                    break;

                case 319:
                    strcpy(buttonText, "T\0");
                    break;

                case 320:
                    strcpy(buttonText, "U\0");
                    break;

                case 321:
                    strcpy(buttonText, "V\0");
                    break;

                case 322:
                    strcpy(buttonText, "W\0");
                    break;

                case 323:
                    strcpy(buttonText, "X\0");
                    break;

                case 324:
                    strcpy(buttonText, "Y\0");
                    break;

                case 325:
                    strcpy(buttonText, "Z\0");
                    break;

                case 326:
                    strcpy(buttonText, "a\0");
                    break;

                case 327:
                    strcpy(buttonText, "b\0");
                    break;

                case 328:
                    strcpy(buttonText, "c\0");
                    break;

                case 329:
                    strcpy(buttonText, "d\0");
                    break;

                case 330:
                    strcpy(buttonText, "e\0");
                    break;

                case 331:
                    strcpy(buttonText, "f\0");
                    break;

                case 332:
                    strcpy(buttonText, "g\0");
                    break;

                case 333:
                    strcpy(buttonText, "h\0");
                    break;

                case 334:
                    strcpy(buttonText, "i\0");
                    break;

                case 335:
                    strcpy(buttonText, "j\0");
                    break;

                case 336:
                    strcpy(buttonText, "k\0");
                    break;

                case 337:
                    strcpy(buttonText, "l\0");
                    break;

                case 338:
                    strcpy(buttonText, "m\0");
                    break;

                case 339:
                    strcpy(buttonText, "n\0");
                    break;

                case 340:
                    strcpy(buttonText, "o\0");
                    break;

                case 341:
                    strcpy(buttonText, "p\0");
                    break;

                case 342:
                    strcpy(buttonText, "q\0");
                    break;

                case 343:
                    strcpy(buttonText, "r\0");
                    break;

                case 344:
                    strcpy(buttonText, "s\0");
                    break;

                case 345:
                    strcpy(buttonText, "t\0");
                    break;

                case 346:
                    strcpy(buttonText, "u\0");
                    break;

                case 347:
                    strcpy(buttonText, "v\0");
                    break;

                case 348:
                    strcpy(buttonText, "w\0");
                    break;

                case 349:
                    strcpy(buttonText, "x\0");
                    break;

                case 350:
                    strcpy(buttonText, "y\0");
                    break;

                case 351:
                    strcpy(buttonText, "z\0");
                    break;


                case 352:
                    strcpy(buttonText, "!\0");//Russian letter 27...
                    break;

                case 353:
                    strcpy(buttonText, "@\0");//Russian letter 28...
                    break;

                case 354:
                    strcpy(buttonText, "#\0");//Russian letter 29...
                    break;

                case 355:
                    strcpy(buttonText, "$\0");//Russian letter 30...
                    break;

                case 356:
                    strcpy(buttonText, "%\0");//Russian letter 31...
                    break;

                case 357:
                    strcpy(buttonText, "^\0");//Russian letter 32...
                    break;

                case 358:
                    strcpy(buttonText, "&\0");//Russian letter 33...
                    break;


                case 359:
                    strcpy(buttonText, "0\0");
                    break;

                case 360:
                    strcpy(buttonText, "1\0");
                    break;

                case 361:
                    strcpy(buttonText, "2\0");
                    break;

                case 362:
                    strcpy(buttonText, "3\0");
                    break;

                case 363:
                    strcpy(buttonText, "4\0");
                    break;

                case 364:
                    strcpy(buttonText, "5\0");
                    break;

                case 365:
                    strcpy(buttonText, "6\0");
                    break;

                case 366:
                    strcpy(buttonText, "7\0");
                    break;

                case 367:
                    strcpy(buttonText, "8\0");
                    break;

                case 368:
                    strcpy(buttonText, "9\0");
                    break;

                case 369:
                    strcpy(buttonText, "+\0");
                    break;

                case 370:
                    strcpy(buttonText, "_\0");
                    break;

                case 371:
                    strcpy(buttonText, "<\0");
                    break;

                case 372:
                    strcpy(buttonText, "End\0");
                    break;


                default:
                    strcpy(buttonText, " \0");
                    break;
            }

            SDL_Color textColor = { 0, 0, 0, 255 };
            SDL_Color outlineColor = { 208, 132, 17, 255 };
            SDL_Surface *text = NULL;
            SDL_Surface *textOutline = NULL;
            SDL_Rect destRect;

            if (index != 372)
            {
                text = TTF_RenderText_Blended(Font[0], buttonText, textColor);
                textOutline = TTF_RenderText_Solid(Font[0], buttonText, outlineColor);
            }
            else if (index == 372)
            {
                text = TTF_RenderText_Blended(Font[2], buttonText, textColor);
                textOutline = TTF_RenderText_Solid(Font[2], buttonText, outlineColor);
            }

            for (int posY = -2; posY < 3; posY++)
            {
                for (int posX = -2; posX < 3; posX++)
                {
                    destRect.x = (tempSurface->w / 2) - (text->w / 2) + posX;
                    destRect.y = (tempSurface->h / 2) - (text->h / 2) + posY;

                    SDL_BlitSurface(textOutline, NULL, tempSurface, &destRect);
                }
            }

            destRect.x = (tempSurface->w / 2) - (text->w / 2);
            destRect.y = (tempSurface->h / 2) - (text->h / 2);

            SDL_BlitSurface(text, NULL, tempSurface, &destRect);

            SDL_FreeSurface(text);
            SDL_FreeSurface(textOutline);
        }

        if (filePath[0] != '~')
        {
            Sprites[index].Texture = SDL_CreateTextureFromSurface(Renderer, tempSurface);

            Sprites[index].ScreenX = 320;
            Sprites[index].ScreenY = 240;
            Sprites[index].ScaleX = 1.0;
            Sprites[index].ScaleY = 1.0;
            Sprites[index].RotationDegree = 0;
            Sprites[index].RedHue = 255;
            Sprites[index].GreenHue = 255;
            Sprites[index].BlueHue = 255;
            Sprites[index].Transparency = 255;
            Sprites[index].Smooth = false;
            Sprites[index].FlipX = false;
            Sprites[index].FlipY = false;

            Sprites[index].AnimationTimer = 0;

            SDL_QueryTexture(Sprites[index].Texture, &textureFormat, &textureAccess, &textureWidth, &textureHeight);
            Sprites[index].TextureWidthOriginal = textureWidth;
            Sprites[index].TextureHeightOriginal = textureHeight;

            SDL_FreeSurface(tempSurface);
        }
    }

    return(true);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadStaffTextIntoMemory(const char *staffText, Uint8 textBlue)
{
Uint32 textureFormat;
int textureAccess;
int textureWidth;
int textureHeight;

    Sprites[1100+TotalNumberOfLoadedStaffTexts].Texture = NULL;

    SDL_Color textColor = { 255, 255, 255, 255 };
    SDL_Color outlineColor = { 0, 0, 0, 255 };
    SDL_Surface *text;
    SDL_Surface *textOutline;
    SDL_Rect destRect;

    SDL_Surface *tempSurface = IMG_Load("data/visuals/Staff-Text-Image.png");

    if (!tempSurface)
    {
        printf( "Image loading failed: %s\n", IMG_GetError() );
        CoreFailure = true;
        return(false);
    }

    text = TTF_RenderText_Blended(Font[1], staffText, textColor);
    textOutline = TTF_RenderText_Solid(Font[1], staffText, outlineColor);

    for (int posY = -2; posY < 3; posY++)
    {
        for (int posX = -2; posX < 3; posX++)
        {
            destRect.x = (tempSurface->w / 2) - (text->w / 2) + posX;
            destRect.y = (tempSurface->h / 2) - (text->h / 2) + posY;

            SDL_BlitSurface(textOutline, NULL, tempSurface, &destRect);
        }
    }

    destRect.x = (tempSurface->w / 2) - (text->w / 2);
    destRect.y = (tempSurface->h / 2) - (text->h / 2);

    SDL_BlitSurface(text, NULL, tempSurface, &destRect);

    SDL_FreeSurface(text);
    SDL_FreeSurface(textOutline);

    Sprites[1100+TotalNumberOfLoadedStaffTexts].Texture = SDL_CreateTextureFromSurface(Renderer, tempSurface);

    Sprites[1100+TotalNumberOfLoadedStaffTexts].ScreenX = 320;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].ScreenY = 240;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].ScaleX = 1.0;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].ScaleY = 1.0;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].RotationDegree = 0;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].RedHue = 255;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].GreenHue = 255;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].BlueHue = textBlue;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].Transparency = 255;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].Smooth = false;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].FlipX = false;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].FlipY = false;

    Sprites[1100+TotalNumberOfLoadedStaffTexts].AnimationTimer = 0;

    SDL_QueryTexture(Sprites[1100+TotalNumberOfLoadedStaffTexts].Texture, &textureFormat, &textureAccess, &textureWidth, &textureHeight);
    Sprites[1100+TotalNumberOfLoadedStaffTexts].TextureWidthOriginal = textureWidth;
    Sprites[1100+TotalNumberOfLoadedStaffTexts].TextureHeightOriginal = textureHeight;

    SDL_FreeSurface(tempSurface);

    TotalNumberOfLoadedStaffTexts++;
    return(true);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::PreloadStaffTextsIntoMemory(void)
{
char textToDisplay[100];
char *copyright = new char[4];
char *reg = new char[4];
//copyright = "";
//reg = "";

strcpy(copyright, "");
strcpy(reg, "");

//sprintf_s(copyright, "%c", 0xA9);
//sprintf_s(reg, "%c", 0xAE);

    if ( LoadStaffTextIntoMemory("TM", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("LettersFall", 0) == false)  return(false);
    strcpy(textToDisplay, copyright);
    strcat(textToDisplay, "2015, By 16BitSoft Inc.");
    if ( LoadStaffTextIntoMemory(textToDisplay, 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Original Game Concept By:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''Merkredy''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''GT-R Twin TurboCharged'' Game Engine Programmer:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Lead Game Designer:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Lead Game Programmer:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Lead Game Tester:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Lead Graphic Artist:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Lead Music Composer/Remixer / Sound Editor:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''D.J. Fading Twilight''", 255) == false)  return(false);
    strcpy(textToDisplay, "Microsoft");
    strcat(textToDisplay, reg);
    strcat(textToDisplay, " Windows");
    strcat(textToDisplay, reg);
    strcat(textToDisplay, " Technical Advisors:");
    if ( LoadStaffTextIntoMemory(textToDisplay, 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''Daotheman''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Linux Technical Advisors:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''mattmatteh''", 255) == false)  return(false);
    strcpy(textToDisplay, "Apple");
    strcat(textToDisplay, reg);
    strcat(textToDisplay, " Mac OS X");
    strcat(textToDisplay, reg);
    strcat(textToDisplay, " Technical Advisor:");
    if ( LoadStaffTextIntoMemory(textToDisplay, 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Someone?", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Email ''Admin@16BitSoft.com'' To Help!", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Support Designers/Programmers/Testers:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''Daotheman''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''mattmatteh''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Support Game Beta Testers:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''You!''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Developed On A:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''JeZxLee'' Pro-Built Desktop: ''Optimus Prime''", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("(Running Genuine Linux Mint 17.1 KDE 64Bit Linux)", 255) == false)  return(false);

    if ( LoadStaffTextIntoMemory("Game Engine Optimized On A:", 0) == false)  return(false);
    strcpy(textToDisplay, "Zotac");
    strcat(textToDisplay, reg);
    strcat(textToDisplay, " ZBox Thin Client Desktop");
    if ( LoadStaffTextIntoMemory(textToDisplay, 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("(Running Genuine Kubuntu 14.10 64Bit Linux)", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("Technology Credits:", 0) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''SDL'' Version 2.0 - Simple DirectMedia Layer", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("(SDL2_Image / SDL2_Mixer / SDL2_TTF)", 255) == false)  return(false);
    if ( LoadStaffTextIntoMemory("''A 110% By JeZxLee''", 0) == false)  return(false);

    return(true);
}

//-------------------------------------------------------------------------------------------------
void Visuals::DrawSpriteOntoScreenBuffer(Uint16 index)
{
	SDL_Rect destinationRect;
	int windowWidth;
	int windowHeight;
	Uint32 textureFormat;
	int textureAccess;
	int textureWidth;
	int textureHeight;

	SDL_GetWindowSize(Window, &windowWidth, &windowHeight);

	SDL_QueryTexture(Sprites[index].Texture, &textureFormat, &textureAccess, &textureWidth, &textureHeight);

	float winWidthFixed;
	float winHeightFixed;
	if (ForceAspectRatio == false)
	{
		winWidthFixed = (float)((float)windowWidth / 640);
		winHeightFixed = (float)((float)windowHeight / 480);
	}
	else
	{
		winWidthFixed = 1;
		winHeightFixed = 1;
	}

	destinationRect.x = (int)( (Sprites[index].ScreenX * (winWidthFixed))
		- (((textureWidth * Sprites[index].ScaleX) * (winWidthFixed)) / 2) );
	destinationRect.y = (int)( (Sprites[index].ScreenY * (winHeightFixed))
		- (((textureHeight * Sprites[index].ScaleY) * (winHeightFixed)) / 2) );
	destinationRect.w = (int)( textureWidth * Sprites[index].ScaleX * (winWidthFixed) );
	destinationRect.h = (int)( textureHeight * Sprites[index].ScaleY * (winHeightFixed) );

	SDL_SetTextureColorMod(Sprites[index].Texture, Sprites[index].RedHue, Sprites[index].GreenHue, Sprites[index].BlueHue);
	SDL_SetTextureAlphaMod(Sprites[index].Texture, Sprites[index].Transparency);

	if (Sprites[index].FlipX == false && Sprites[index].FlipY == false)
	{
		SDL_RenderCopyEx(Renderer, Sprites[index].Texture, NULL, &destinationRect, Sprites[index].RotationDegree
			, NULL, SDL_FLIP_NONE);
	}
	else if (Sprites[index].FlipX == true && Sprites[index].FlipY == false)
	{
		SDL_RenderCopyEx(Renderer, Sprites[index].Texture, NULL, &destinationRect, Sprites[index].RotationDegree
			, NULL, SDL_FLIP_HORIZONTAL);
	}
	else if (Sprites[index].FlipX == false && Sprites[index].FlipY == true)
	{
		SDL_RenderCopyEx(Renderer, Sprites[index].Texture, NULL, &destinationRect, Sprites[index].RotationDegree
			, NULL, SDL_FLIP_VERTICAL);
	}
	else if (Sprites[index].FlipX == true && Sprites[index].FlipY == true)
	{
		double flipHorizontallyAndVerticallyDegreeFix = Sprites[index].RotationDegree + 180;

		SDL_RenderCopyEx(Renderer, Sprites[index].Texture, NULL, &destinationRect, flipHorizontallyAndVerticallyDegreeFix
			, NULL, SDL_FLIP_NONE);
	}
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadFontsIntoMemory(void)
{
    Font[0] = NULL;
    if ( !(Font[0] = TTF_OpenFont("data/fonts/Font-01.ttf", 27)) )
    {
        printf( "Error loading font: %s\n", TTF_GetError() );
        CoreFailure = true;
        return false;
    }

    Font[1] = NULL;
    if ( !(Font[1] = TTF_OpenFont("data/fonts/Font-01.ttf", 20)) )
    {
        printf( "Error loading font: %s\n", TTF_GetError() );
        CoreFailure = true;
        return false;
    }

    Font[2] = NULL;
    if ( !(Font[2] = TTF_OpenFont("data/fonts/Font-01.ttf", 14)) )
    {
        printf( "Error loading font: %s\n", TTF_GetError() );
        CoreFailure = true;
        return false;
    }

    Font[3] = NULL;
    if ( !(Font[3] = TTF_OpenFont("data/fonts/Line-01.ttf", 26)) )
    {
        printf( "Error loading font: %s\n", TTF_GetError() );
        CoreFailure = true;
        return false;
    }

    Font[4] = NULL;
    if ( !(Font[4] = TTF_OpenFont("data/fonts/Font-01.ttf", 55)) )
    {
        printf( "Error loading font: %s\n", TTF_GetError() );
        CoreFailure = true;
        return false;
    }

    printf("Loaded all TTF fonts into memory.\n");

    return true;
}

//-------------------------------------------------------------------------------------------------
void Visuals::UnloadFontsFromMemory(void)
{
    for (int index = 0; index < NumberOfFonts; index++)
    {
        if (Font[index] != NULL)  TTF_CloseFont(Font[index]);
    }
    printf("Unloaded all TTF fonts from memory.\n");
}

//-------------------------------------------------------------------------------------------------
void Visuals::ClearTextCache(void)
{
    for (int index = 0; index < NumberOfTextsCached; index++)
    {
        if (TextTexture[index] != NULL)  SDL_DestroyTexture(TextTexture[index]);
        if (TextOutlineTexture[index] != NULL)  SDL_DestroyTexture(TextOutlineTexture[index]);

        TextTexture[index] = NULL;
        TextOutlineTexture[index] = NULL;

        TextCachedText[index][0] = '\0';
        TextCachedScreenX[index] = 320;
        TextCachedScreenY[index] = 240;

        TextCachedWidth[index] = 0;
        TextCachedHeight[index] = 0;
    }

    TextCacheCurrentIndex = 0;

    printf("Cleared TTF text cache.\n");
}

//-------------------------------------------------------------------------------------------------
void Visuals::DrawTextOntoScreenBuffer(const char *textToDisplay, TTF_Font *font, int posX, int posY
	, Uint8 XJustification, Uint8 textRed, Uint8 textGreen, Uint8 textBlue
	, Uint8 outlineRed, Uint8 outlineGreen, Uint8 outlineBlue)
{
	SDL_Color textColor = { textRed, textGreen, textBlue, 255 };
	SDL_Color outlineColor = { outlineRed, outlineGreen, outlineBlue, 255 };
	SDL_Surface *text = NULL;
	SDL_Surface *textOutline = NULL;
	SDL_Texture *textTexture = NULL;
	SDL_Texture *textOutlineTexture = NULL;
	SDL_Rect destinationRect;
	SDL_Rect destinationOutlineRect;
	int windowWidth;
	int windowHeight;
	int textCacheCheckIndex = NumberOfTextsCached;
	bool sentenceMatches = true;
	bool sentenceIsInCache = false;

	for (textCacheCheckIndex = 0; textCacheCheckIndex < NumberOfTextsCached; textCacheCheckIndex++)
	{
		sentenceMatches = true;
		if (strlen(TextCachedText[textCacheCheckIndex]) == strlen(textToDisplay))
		{
			for (Uint16 index = 0; index < strlen(textToDisplay); index++)
			{
				if (TextCachedText[textCacheCheckIndex][index] != textToDisplay[index])  sentenceMatches = false;
			}
		}
		else  sentenceMatches = false;

		if (sentenceMatches == true)
		{
			if (TextCachedScreenX[textCacheCheckIndex] == posX && TextCachedScreenY[textCacheCheckIndex] == posY)
			{
				sentenceIsInCache = true;
				break;
			}
		}
	}

	if (sentenceIsInCache == false)
	{
		if (TextTexture[TextCacheCurrentIndex] != NULL)  SDL_DestroyTexture(TextTexture[TextCacheCurrentIndex]);
		if (TextOutlineTexture[TextCacheCurrentIndex] != NULL)  SDL_DestroyTexture(TextOutlineTexture[TextCacheCurrentIndex]);
		TextTexture[TextCacheCurrentIndex] = NULL;
		TextOutlineTexture[TextCacheCurrentIndex] = NULL;
		TextCachedText[TextCacheCurrentIndex][0] = '\0';
		TextCachedScreenX[TextCacheCurrentIndex] = 320;
		TextCachedScreenY[TextCacheCurrentIndex] = 240;
		TextCachedWidth[TextCacheCurrentIndex] = 0;
		TextCachedHeight[TextCacheCurrentIndex] = 0;

		text = TTF_RenderText_Blended(font, textToDisplay, textColor);
		textOutline = TTF_RenderText_Blended(font, textToDisplay, outlineColor);

		strcpy(TextCachedText[TextCacheCurrentIndex], textToDisplay);

		printf("(");
		if (TextCacheCurrentIndex < 10)  printf("0");
		printf("%i", TextCacheCurrentIndex);
		printf(")");

		for (Uint16 index = 0; index < strlen(TextCachedText[TextCacheCurrentIndex]); index++)
		{
			printf("%c", TextCachedText[TextCacheCurrentIndex][index]);
		}

		printf(" | ");
		for (Uint16 index = 0; index < strlen(textToDisplay); index++)
		{
			printf("%c", textToDisplay[index]);
		}
		printf("\n");

		TextCachedScreenX[TextCacheCurrentIndex] = posX;
		TextCachedScreenY[TextCacheCurrentIndex] = posY;

		TextCachedWidth[TextCacheCurrentIndex] = text->w;
		TextCachedHeight[TextCacheCurrentIndex] = text->h;

		textTexture = SDL_CreateTextureFromSurface(Renderer, text);
		TextTexture[TextCacheCurrentIndex] = SDL_CreateTextureFromSurface(Renderer, text);

		textOutlineTexture = SDL_CreateTextureFromSurface(Renderer, textOutline);
		TextOutlineTexture[TextCacheCurrentIndex] = SDL_CreateTextureFromSurface(Renderer, textOutline);

		if (XJustification == JustifyLeft)
		{
			posX = posX + (text->w / 2);
		}
		else if (XJustification == JustifyCenter)
		{
			posX = (640 / 2);
		}
		else if (XJustification == JustifyRight)
		{
			posX = (640 - posX) - (text->w / 2);
		}
		else if (XJustification == JustifyCenterOnPoint)
		{
			posX = posX;
		}

		if (TextCacheCurrentIndex < (NumberOfTextsCached - 1))  TextCacheCurrentIndex++;
		else  TextCacheCurrentIndex = 0;
	}
	else
	{
		if (XJustification == JustifyLeft)
		{
			posX = posX + (TextCachedWidth[textCacheCheckIndex] / 2);
		}
		else if (XJustification == JustifyCenter)
		{
			posX = (640 / 2);
		}
		else if (XJustification == JustifyRight)
		{
			posX = (640 - posX) - (TextCachedWidth[textCacheCheckIndex] / 2);
		}
		else if (XJustification == JustifyCenterOnPoint)
		{
			posX = posX;
		}
	}

	SDL_GetWindowSize(Window, &windowWidth, &windowHeight);

	float winWidthFixed;
	float winHeightFixed;
	if (ForceAspectRatio == false)
	{
		winWidthFixed = (float)((float)windowWidth / 640);
		winHeightFixed = (float)((float)windowHeight / 480);
	}
	else
	{
		winWidthFixed = 1;
		winHeightFixed = 1;
	}

	if (sentenceIsInCache == false)
	{
		destinationRect.x = (int)( (posX * winWidthFixed) - ((text->w * winWidthFixed) / 2) );
		destinationRect.y = (int)( (posY * winHeightFixed) - (winHeightFixed / 2) + 3 );
		destinationRect.w = (int)( text->w * (winWidthFixed) );
		destinationRect.h = (int)( text->h * (winHeightFixed) );
	}
	else
	{
		destinationRect.x = (int)( (posX * winWidthFixed) - ((TextCachedWidth[textCacheCheckIndex] * winWidthFixed) / 2) );
		destinationRect.y = (int)( (posY * winHeightFixed) - (winHeightFixed / 2) + 3 );
		destinationRect.w = (int)( TextCachedWidth[textCacheCheckIndex] * (winWidthFixed) );
		destinationRect.h = (int)( TextCachedHeight[textCacheCheckIndex] * (winHeightFixed) );
	}

	destinationOutlineRect.x = destinationRect.x;
	destinationOutlineRect.y = destinationRect.y;
	destinationOutlineRect.w = destinationRect.w;
	destinationOutlineRect.h = destinationRect.h;

	for (Sint16 y = -3; y < 4; y += 1)
	{
		for (Sint16 x = -3; x < 4; x += 1)
		{
			destinationOutlineRect.x = destinationRect.x + x;
			destinationOutlineRect.y = destinationRect.y + y;

			if (sentenceIsInCache == false)
				SDL_RenderCopyEx(Renderer, textOutlineTexture, NULL, &destinationOutlineRect, 0, NULL, SDL_FLIP_NONE);
			else
			{
				SDL_RenderCopyEx(Renderer, TextOutlineTexture[textCacheCheckIndex], NULL, &destinationOutlineRect, 0, NULL, SDL_FLIP_NONE);
			}
		}
	}

	if (sentenceIsInCache == false)
		SDL_RenderCopyEx(Renderer, textTexture, NULL, &destinationRect, 0, NULL, SDL_FLIP_NONE);
	else
		SDL_RenderCopyEx(Renderer, TextTexture[textCacheCheckIndex], NULL, &destinationRect, 0, NULL, SDL_FLIP_NONE);

	SDL_DestroyTexture(textOutlineTexture);
	SDL_DestroyTexture(textTexture);
	SDL_FreeSurface(text);
	SDL_FreeSurface(textOutline);
}

//-------------------------------------------------------------------------------------------------
